function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const Promise = require('bluebird');
const HydraPlugin = require('hydra-plugin');
const R = require('ramda');
const uuid = require('uuid/v1');
const didyoumean = require('didyoumean');

const rpcKey = 'hydra:service:rpc:hash';

class HydraRPC extends HydraPlugin {
	constructor() {
		super('hydra-plugin-rpc');
	}

	setHydra(hydra) {
		super.setHydra(hydra);

		let initialized = false;
		const initRPC = () => {
			if (!hydra.isService) {
				hydra.registerService();
			}
			if (!initialized) {
				initialized = true;
				const sub = hydra.redisdb.duplicate();
				sub.on('message', (() => {
					var _ref = _asyncToGenerator(function* (channel, message) {
						const { id, methodName, args } = JSON.parse(message);
						const handler = R.propOr(R.always(null), methodName, hydra._rpcHandlers);
						const data = yield handler.apply(handler, args);
						hydra.redisdb.publish(id, JSON.stringify({ data }));
					});

					return function (_x, _x2) {
						return _ref.apply(this, arguments);
					};
				})());
				sub.subscribe(`hydra-plugin-rpc:${hydra.getInstanceID()}`);
			}
		};

		hydra.methods = obj => R.toPairs(obj).forEach(([methodName, fn]) => addMethod(methodName, fn));

		const addMethod = (methodName, fn) => {
			hydra.redisdb.hset(rpcKey, methodName, hydra.serviceName);
			hydra._rpcHandlers = hydra._rpcHandlers || {};
			hydra._rpcHandlers[methodName] = fn;
			initRPC();
		};

		const rand = (min, max) => Math.floor(Math.random() * (max - min)) + min;

		hydra.call = (() => {
			var _ref2 = _asyncToGenerator(function* (methodName, ...args) {
				const id = uuid();
				const serviceName = yield hydra.redisdb.hgetAsync(rpcKey, methodName);
				const instanceIds = R.map(R.prop('instanceID'), (yield hydra.getServicePresence(serviceName)));
				if (!serviceName || instanceIds.length == 0) {
					const allMethods = R.keys((yield hydra.redisdb.hgetallAsync(rpcKey)));
					const closest = didyoumean(methodName, allMethods);
					const suggest = closest ? `Did you mean "${closest}"?` : '';
					throw new Error(`No Service registered for "${methodName}" method. ${suggest}`);
				}
				const publish = function (instance) {
					return hydra.redisdb.publishAsync(`hydra-plugin-rpc:${instance}`, JSON.stringify({ id, methodName, args }));
				};
				const result = new Promise(function (resolve, reject) {
					const sub = hydra.redisdb.duplicate();
					sub.on('subscribe', _asyncToGenerator(function* () {
						let count = 0,
						    tries = 0;
						while (count == 0 && tries < 20) {
							count = yield publish(instanceIds[rand(0, instanceIds.length)]);
							tries++;
						}
						if (count == 0) {
							reject(new Error('No available endpoints'));
						}
					}));
					sub.on('message', function (channel, message) {
						resolve(JSON.parse(message).data);
						sub.unsubscribe();
						sub.quit();
					});
					sub.subscribe(id);
				});
				return result;
			});

			return function (_x3) {
				return _ref2.apply(this, arguments);
			};
		})();
	}

	onServiceReady() {
		Promise.promisifyAll(this.hydra.redisdb);
	}
}
module.exports = HydraRPC;